# Amethyst - An experimental Minecraft server written in Elixir.
# Copyright (C) 2024 KodiCraft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

defmodule Amethyst.Data do
  @type _boolean :: bool()
  def is?({:_boolean}, x), do: is_boolean(x)
  @type _byte :: -128..127
  def is?({:_byte}, x), do: is_integer(x) and x in -128..127
  @type _ubyte :: 0..255
  def is?({:_ubyte}, x), do: is_integer(x) and x in 0..255
  @type _short :: -32768..32767
  def is?({:_short}, x), do: is_integer(x) and x in -32768..32767
  @type _ushort :: 0..65535
  def is?({:_ushort}, x), do: is_integer(x) and x in 0..65535
  @type _int :: -2147483648..2147483647
  def is?({:_int}, x), do: is_integer(x) and x in -2147483648..2147483647
  @type _uint :: 0..4294967295
  def is?({:_uint}, x), do: is_integer(x) and x in 0..4294967295
  @type _long :: -9223372036854775808..9223372036854775807
  def is?({:_long}, x), do: is_integer(x) and x in -9223372036854775808..9223372036854775807
  @type _float :: float()
  def is?({:_float}, x), do: is_float(x)
  @type _double :: float()
  def is?({:_double}, x), do: is_float(x)
  @type _string :: String.t()
  def is?({:_string}, x), do: is_binary(x)
  @type _nbt :: map() # TODO: Type-ify NBT
  def is?({:_nbt}, x), do: is_map(x)
  @type _varint :: _int()
  def is?({:_varint}, x), do: is?({:_int}, x)
  @type _varlong :: _long()
  def is?({:_varlong}, x), do: is?({:_long}, x)
  @type _position :: {-33554432..33554431, -2048..2047, -33554432..33554431}
  def is?({:_position}, {x, y, z}), do: is?({:_int}, x) and is?({:_int}, y) and is?({:_int}, z)
  @type _uuid :: UUID.uuid4()
  def is?({:_uuid}, x), do: is_binary(x) and byte_size(x) == 16 # Not the best check but it's good enoug
  @type _bitset :: [boolean()]
  def is?({:_bitset}, x), do: is_list(x) and Enum.all?(x, &is_boolean/1)
  def is?({:_bitset, n}, x), do: is_list(x) and Enum.all?(x, &is_boolean/1) and length(x) == n
  @type _optional(n) :: n | nil
  def is?({:_optional, n}, x), do: is?({n}, x) or is_nil(x)
  @type _array(n) :: [n]
  def is?({:_array, n}, x), do: is_list(x) and Enum.all?(x, &is?({n}, &1))
end
